@@ -1,10 +1,13 @@
 package Lemonldap::NG::Manager::Conf::Tests;
 
+use strict;
 use utf8;
+use strict;
 use Lemonldap::NG::Common::Regexp;
 use Lemonldap::NG::Handler::Main;
+use Lemonldap::NG::Common::Util qw(getSameSite);
 
-our $VERSION = '2.0.7';
+our $VERSION = '2.0.10';
 
 ## @method hashref tests(hashref conf)
 # Return a hash ref where keys are the names of the tests and values
@@ -42,14 +45,8 @@ sub tests {
         # Check if portal URL is well formated
         portalURL => sub {
 
-            # Checking for ending slash
-            $conf->{portal} .= '/'
-              unless ( $conf->{portal} =~ qr#/$# );
-
-            # Deleting trailing ending slash
-            my $regex = qr#/+$#;
-            $conf->{portal} =~ s/$regex/\//;
-
+            # Append or remove trailing ending slashes
+            $conf->{portal} =~ s%/*$%/%;
             return 1;
         },
 
@@ -231,7 +228,7 @@ sub tests {
             eval { tied(%h)->delete; };
             return ( -1, "Unable to delete session ($@)" ) if ($@);
             return ( -1,
-'All sessions may be lost and you must restart all your Apache servers'
+'All sessions may be lost and you must restart all your web servers'
             ) if ( $gc and $conf->{globalStorage} ne $gc );
             return 1;
         },
@@ -259,8 +256,6 @@ sub tests {
             return ( 1, "Cookie TTL should be higher or equal than one hour" )
               unless ( $conf->{cookieExpiration} >= 3600
                 || $conf->{cookieExpiration} == 0 );
-
-            # Return
             return 1;
         },
 
@@ -270,8 +265,6 @@ sub tests {
             return ( -1, "Session timeout should be higher than ten minutes" )
               unless ( $conf->{timeout} > 600
                 || $conf->{timeout} == 0 );
-
-            # Return
             return 1;
         },
 
@@ -283,8 +276,6 @@ sub tests {
               )
               unless ( $conf->{timeoutActivity} > 59
                 || $conf->{timeoutActivity} == 0 );
-
-            # Return
             return 1;
         },
 
@@ -297,8 +288,6 @@ sub tests {
               if (  $conf->{timeoutActivity}
                 and $conf->{timeoutActivity} <=
                 $conf->{timeoutActivityInterval} );
-
-            # Return
             return 1;
         },
 
@@ -314,41 +303,43 @@ sub tests {
             );
         },
 
+        # Test support of timeouts for LDAPS connections
+        ldapsNoTimeout => sub {
+
+            # Skip test if no SMTP configuration
+            return (1) unless ( $conf->{ldapServer} );
+
+            if ( $conf->{ldapServer} =~ /ldaps:/ ) {
+
+                if ( eval "require IO::Socket::SSL; require IO::Socket::IP;" ) {
+                    if ( IO::Socket::SSL->isa('IO::Socket::IP') ) {
+                        unless ( eval { IO::Socket::IP->VERSION(0.31) } ) {
+                            return ( 1,
+"Your version of IO::Socket::IP is too old to enforce "
+                                  . "connection timeouts on ldaps:// URLs. Use ldap+tls:// instead"
+                            );
+                        }
+                    }
+                }
+            }
+            return (1);
+        },
+
         # Test SMTP connection and authentication (warning only)
-        smtpConnectionAuthentication => sub {
+        smtpConfiguration => sub {
 
             # Skip test if no SMTP configuration
             return 1 unless ( $conf->{SMTPServer} );
 
             # Use SMTP
-            eval "use Net::SMTP";
-            return ( 1, "Net::SMTP module is required to use SMTP server" )
+            eval "use Lemonldap::NG::Common::EmailTransport";
+            return ( 1, "Could not load Lemonldap::NG::Common::EmailTransport" )
               if ($@);
 
-            # Create SMTP object
-            my $smtp = Net::SMTP->new(
-                $conf->{SMTPServer},
-                Timeout => 5,
-                ( $conf->{SMTPPort} ? ( Port => $conf->{SMTPPort} ) : () ),
-            );
-            return ( 1,
-                "SMTP connection to " . $conf->{SMTPServer} . " failed" )
-              unless ($smtp);
-
-            # Skip other tests if no authentication
-            return 1
-              unless ( $conf->{SMTPAuthUser} and $conf->{SMTPAuthPass} );
-
-            # Try authentication
-            return ( 1, "SMTP authentication failed" )
-              unless $smtp->auth( $conf->{SMTPAuthUser},
-                $conf->{SMTPAuthPass} );
-
-            # Return
-            return 1;
+            return Lemonldap::NG::Common::EmailTransport->configTest($conf);
         },
 
-        # SAML entity ID must be uniq
+        # SAML entity ID must be unique
         samlIDPEntityIdUniqueness => sub {
             return 1
               unless ( $conf->{samlIDPMetaDataXML}
@@ -414,6 +405,76 @@ sub tests {
             return 1;
         },
 
+        samlSignatureOverrideNeedsCertificate => sub {
+            return 1 if $conf->{samlServicePublicKeySig} =~ /CERTIFICATE/;
+
+            my @offenders;
+            for my $idp ( keys %{ $conf->{samlIDPMetaDataOptions} } ) {
+                if ( $conf->{samlIDPMetaDataOptions}->{$idp}
+                    ->{samlIDPMetaDataOptionsSignatureMethod} )
+                {
+                    push @offenders, $idp;
+                }
+            }
+            for my $sp ( keys %{ $conf->{samlSPMetaDataOptions} } ) {
+                if ( $conf->{samlSPMetaDataOptions}->{$sp}
+                    ->{samlSPMetaDataOptionsSignatureMethod} )
+                {
+                    push @offenders, $sp;
+                }
+            }
+            return @offenders
+              ? (
+                0,
+                "Cannot set non-default signature method on "
+                  . join( ", ", @offenders )
+                  . " unless SAML signature key is in certificate form"
+              )
+              : 1;
+        },
+
+        samlSignatureUnsupportedAlg => sub {
+            return 1
+              unless eval
+'use Lasso; Lasso::check_version( 2, 5, 1, Lasso::Constants::CHECK_VERSION_NUMERIC) ? 0:1';
+
+            my $allsha1 = 1;
+            undef $allsha1
+              unless $conf->{samlServiceSignatureMethod} eq "RSA_SHA1";
+
+            for my $idp ( keys %{ $conf->{samlIDPMetaDataOptions} } ) {
+                if ( $conf->{samlIDPMetaDataOptions}->{$idp}
+                    ->{samlIDPMetaDataOptionsSignatureMethod} )
+                {
+                    if ( $conf->{samlIDPMetaDataOptions}->{$idp}
+                        ->{samlIDPMetaDataOptionsSignatureMethod} ne
+                        "RSA_SHA1" )
+                    {
+                        undef $allsha1;
+                        last;
+                    }
+                }
+            }
+            for my $sp ( keys %{ $conf->{samlSPMetaDataOptions} } ) {
+                if ( $conf->{samlSPMetaDataOptions}->{$sp}
+                    ->{samlSPMetaDataOptionsSignatureMethod} )
+                {
+                    if ( $conf->{samlSPMetaDataOptions}->{$sp}
+                        ->{samlSPMetaDataOptionsSignatureMethod} ne "RSA_SHA1" )
+                    {
+                        undef $allsha1;
+                        last;
+                    }
+                }
+            }
+            return $allsha1
+              ? 1
+              : (
+                0,
+                "Algorithms other than SHA1 are only supported on Lasso>=2.5.1"
+              );
+        },
+
         # Try to parse combination with declared modules
         checkCombinations => sub {
             return 1 unless ( $conf->{authentication} eq 'Combination' );
@@ -446,8 +507,6 @@ sub tests {
               unless ( $conf->{combination} );
             return ( 0, 'userDB must be set to "Same" to enable Combination' )
               unless ( $conf->{userDB} eq "Same" );
-
-            # Return
             return 1;
         },
 
@@ -487,8 +546,6 @@ sub tests {
 "Auth::Yubikey_WebClient module is required to enable Yubikey"
                 ) if ($@);
             }
-
-            # Return
             return 1;
         },
 
@@ -526,8 +583,6 @@ sub tests {
               unless ( $conf->{totp2fRange} );
             return ( 1, "TOTP interval should be higher than 10s" )
               unless ( $conf->{totp2fInterval} > 10 );
-
-            # Return
             return 1;
         },
 
@@ -575,7 +630,6 @@ sub tests {
                 || $conf->{'totp2fSelfRegistration'} );
             $msg = "A self registrable module should be enabled to require 2FA"
               unless ($ok);
-
             return ( 1, $msg );
         },
 
@@ -588,8 +642,6 @@ sub tests {
                 return ( 0, "External 2F Validate command must be set" )
                   unless ( defined $conf->{ext2FValidateCommand} );
             }
-
-            # Return
             return 1;
         },
 
@@ -600,20 +652,16 @@ sub tests {
               unless ( $conf->{formTimeout} > 30 );
             return ( 1, "XSRF form token TTL should not be higher than 2mn" )
               if ( $conf->{formTimeout} > 120 );
-
-            # Return
             return 1;
         },
 
         # Warn if issuers token TTL is higher than 30s
         issuersTimeout => sub {
-            return 1 unless ( defined $conf->{issuerTimeout} );
+            return 1 unless ( defined $conf->{issuersTimeout} );
             return ( 0, "Issuers token TTL must be higher than 30s" )
-              unless ( $conf->{issuerTimeout} > 30 );
+              unless ( $conf->{issuersTimeout} > 30 );
             return ( 1, "Issuers token TTL should not be higher than 2mn" )
-              if ( $conf->{issuerTimeout} > 120 );
-
-            # Return
+              if ( $conf->{issuersTimeout} > 120 );
             return 1;
         },
 
@@ -622,35 +670,51 @@ sub tests {
             return 1 unless ( $conf->{portalDisplayResetPassword} );
             return ( 1, "Number of reset password retries should not be null" )
               unless ( $conf->{passwordResetAllowedRetries} );
-
-            # Return
             return 1;
         },
 
         # Warn if ldapPpolicyControl is used with AD (#2007)
-
         ppolicyAd => sub {
-            if (    $conf->{ldapPpolicyControl}
-                and $conf->{authentication} eq "AD" )
-            {
-                return ( 1,
+            return ( 1,
 "LDAP password policy control should be disabled when using AD authentication"
-                );
-            }
+              )
+              if (  $conf->{ldapPpolicyControl}
+                and $conf->{authentication} eq "AD" );
             return 1;
         },
 
         # Warn if bruteForceProtection enabled without History
         bruteForceProtection => sub {
+            my @lockTimes =
+              sort { $a <=> $b }
+              map {
+                $_ =~ s/\D//;
+                abs $_;
+              }
+              grep { /\d+/ }
+              split /\s*,\s*/, $conf->{bruteForceProtectionLockTimes};
+            $conf->{bruteForceProtectionLockTimes} = join ', ', @lockTimes if scalar @lockTimes;
             return 1 unless ( $conf->{bruteForceProtection} );
-            return ( 1,
+            return ( 0,
 '"History" plugin is required to enable "BruteForceProtection" plugin'
             ) unless ( $conf->{loginHistoryEnabled} );
-            return ( 1,
-'Number of failed logins must be higher than 2 to enable "BruteForceProtection" plugin'
-            ) unless ( $conf->{failedLoginNumber} > 2 );
-
-            # Return
+            return ( 0,
+'Number of failed logins must be higher than 1 to enable "BruteForceProtection" plugin'
+            ) unless ( $conf->{failedLoginNumber} > 1 );
+            return ( 0,
+'Number of allowed failed logins must be higher than 0 to enable "BruteForceProtection" plugin'
+            ) unless ( $conf->{bruteForceProtectionMaxFailed} > 0 );
+            return ( 0,
+'Number of failed logins history must be higher or equal than allowed failed logins plus lock time values'
+              )
+              if ( $conf->{bruteForceProtectionIncrementalTempo}
+                && $conf->{failedLoginNumber} <
+                $conf->{bruteForceProtectionMaxFailed} + scalar @lockTimes );
+            return ( 0,
+'Number of failed logins history must be higher or equal than allowed failed logins'
+              )
+              unless ( $conf->{failedLoginNumber} >=
+                $conf->{bruteForceProtectionMaxFailed} );
             return 1;
         },
 
@@ -662,8 +726,6 @@ sub tests {
               )
               unless ( $conf->{requireToken}
                 or $conf->{captcha_mail_enabled} );
-
-            # Return
             return 1;
         },
 
@@ -674,8 +736,6 @@ sub tests {
               )
               if ( $conf->{impersonationRule}
                 && $conf->{contextSwitchingRule} );
-
-            # Return
             return 1;
         },
 
@@ -687,7 +747,8 @@ sub tests {
                 || $conf->{yubikey2fActivation}
                 || $conf->{u2fActivation}
                 || $conf->{utotp2fActivation} );
-            return ( 1, "History enabled WITHOUT persistent session storage" )
+            return ( 1,
+                "History plugin enabled WITHOUT persistent session storage" )
               if ( $conf->{loginHistoryEnabled} );
             return ( 1,
                 "OIDC consents enabled WITHOUT persistent session storage" )
@@ -698,8 +759,6 @@ sub tests {
             return ( 1,
 "BruteForceProtection plugin enabled WITHOUT persistent session storage"
             ) if ( $conf->{bruteForceProtection} );
-
-            # Return
             return 1;
         },
 
@@ -714,8 +773,19 @@ sub tests {
             return ( 1,
 "XML::LibXSLT module is required to enable old format notifications"
             ) if ($@);
+            return 1;
+        },
 
-            # Return
+        # Warn if CertificateResetByMail dependencies seem missing
+        certResetByMailDependencies => sub {
+            return 1 unless ( $conf->{portalDisplayCertificateResetByMail} );
+            return ( 0,
+"LDAP RegisterDB is required to enable CertificateResetByMail plugin"
+            ) unless ( $conf->{registerDB} eq 'LDAP' );
+            eval "use DateTime::Format::RFC3339";
+            return ( 1,
+"DateTime::Format::RFC3339 module is required to enable CertificateResetByMail plugin"
+            ) if ($@);
             return 1;
         },
 
@@ -739,6 +809,133 @@ sub tests {
             return ( $res, join( ', ', @msg ) );
         },
 
+        # RS* OIDC algs require a signing key
+        oidcRPNeedRSAKey => sub {
+            return 1
+              unless ( $conf->{oidcRPMetaDataOptions}
+                and %{ $conf->{oidcRPMetaDataOptions} } );
+            my @usingRSA = grep {
+                $conf->{oidcRPMetaDataOptions}->{$_}
+                  ->{oidcRPMetaDataOptionsIDTokenSignAlg}
+                  and $conf->{oidcRPMetaDataOptions}->{$_}
+                  ->{oidcRPMetaDataOptionsIDTokenSignAlg} =~ /^RS/
+            } keys %{ $conf->{oidcRPMetaDataOptions} };
+
+            if ( @usingRSA and not $conf->{oidcServicePrivateKeySig} ) {
+                my $msg =
+                  join( ", ", @usingRSA )
+                  . ": using RS-type encryption, but no RSA key is defined in global OIDC configuration";
+                return ( 0, $msg );
+            }
+            return 1;
+        },
+
+        # OIDC RP Client ID must exist and be unique
+        oidcRPClientIdUniqueness => sub {
+            return 1
+              unless ( $conf->{oidcRPMetaDataOptions}
+                and %{ $conf->{oidcRPMetaDataOptions} } );
+            my @msg;
+            my $res = 1;
+            my %clientIds;
+            foreach
+              my $clientConfKey ( keys %{ $conf->{oidcRPMetaDataOptions} } )
+            {
+                my $clientId = $conf->{oidcRPMetaDataOptions}->{$clientConfKey}
+                  ->{oidcRPMetaDataOptionsClientID};
+                unless ($clientId) {
+                    push @msg,
+                      "$clientConfKey OIDC Relying Party has no Client ID";
+                    $res = 0;
+                    next;
+                }
+
+                if ( defined $clientIds{$clientId} ) {
+                    push @msg,
+"$clientConfKey and $clientIds{$clientId} have the same Client ID";
+                    $res = 0;
+                    next;
+                }
+                $clientIds{$clientId} = $clientConfKey;
+            }
+            return ( $res, join( ', ', @msg ) );
+        },
+
+        # CAS APP URL must be defined and unique
+        casAppHostnameUniqueness => sub {
+            return 1
+              unless ( $conf->{casAppMetaDataOptions}
+                and %{ $conf->{casAppMetaDataOptions} } );
+            my @msg;
+            my $res = 1;
+            my %casUrl;
+            foreach my $casConfKey ( keys %{ $conf->{casAppMetaDataOptions} } )
+            {
+                my $appUrl =
+                  $conf->{casAppMetaDataOptions}->{$casConfKey}
+                  ->{casAppMetaDataOptionsService}
+                  || "";
+                $appUrl =~ m#^(https?://[^/]+)(/.*)?$#;
+                my $appHost = $1;
+                unless ($appHost) {
+                    push @msg, "$casConfKey CAS Application has no Service URL";
+                    $res = 0;
+                    next;
+                }
+
+                if ( defined $casUrl{$appUrl} ) {
+                    push @msg,
+"$casConfKey and $casUrl{$appUrl} have the same Service URL";
+                    $res = 0;
+                    next;
+                }
+                $casUrl{$appUrl} = $casConfKey;
+            }
+            return ( $res, join( ', ', @msg ) );
+        },
+
+        # Notification system required with removed SF notification
+        sfRemovedNotification => sub {
+            return 1 unless ( $conf->{sfRemovedMsgRule} );
+            return ( 1,
+'Notification system must be enabled to display a notification if a SF is removed'
+            ) if ( $conf->{sfRemovedUseNotif} and not $conf->{notification} );
+            return 1;
+        },
+
+        # noAjaxHook and krbByJs are incompatible (#2237)
+        noAjaxHookwithKrb => sub {
+            return ( 1,
+                    'noAjaxHook is not compatible with'
+                  . ' AJAX Kerberos authentication' )
+              if ( $conf->{noAjaxHook} and $conf->{krbByJs} );
+            return 1;
+        },
+
+        # Cookie SameSite=None requires Secure flag
+        # Same with SameSite=(auto) and SAML issuer in use
+        SameSiteNoneWithSecure => sub {
+            return ( -1, 'SameSite value = None requires the secured flag' )
+              if ( getSameSite($conf) eq 'None' and !$conf->{securedCookie} );
+            return 1;
+        },
+
+        # Secure cookies require HTTPS
+        SecureCookiesRequireHttps => sub {
+            return ( -1, 'Secure cookies require a HTTPS portal URL' )
+              if (  $conf->{securedCookie} == 1
+                and $conf->{portal}
+                and $conf->{portal} !~ /^https:/ );
+            return 1;
+        },
+
+        # Password module requires a password backend
+        passwordModuleNeedsBackend => sub {
+            return ( -1, 'Password module is enabled without password backend' )
+              if (  $conf->{portalDisplayChangePassword}
+                and $conf->{passwordDB} eq 'Null' );
+            return 1;
+        },
     };
 }
 
