@@ -8,13 +8,14 @@
 #                  of lemonldap-ng.ini) and underlying handler configuration
 package Lemonldap::NG::Portal::Main::Init;
 
-our $VERSION = '2.0.7';
+our $VERSION = '2.0.10';
 
 package Lemonldap::NG::Portal::Main;
 
 use strict;
 use Mouse;
 use Regexp::Assemble;
+use Lemonldap::NG::Common::Util qw(getSameSite);
 
 # PROPERTIES
 
@@ -73,6 +74,9 @@ BEGIN {
 has 'afterSub'  => ( is => 'rw', default => sub { {} } );
 has 'aroundSub' => ( is => 'rw', default => sub { {} } );
 
+# Issuer hooks
+has 'hook' => ( is => 'rw', default => sub { {} } );
+
 has spRules => (
     is      => 'rw',
     default => sub { {} }
@@ -87,6 +91,9 @@ has csp => ( is => 'rw' );
 # Cross-Origine Resource Sharing headers
 has cors => ( is => 'rw' );
 
+# Cookie SameSite value
+has cookieSameSite => ( is => 'rw' );
+
 # INITIALIZATION
 
 sub init {
@@ -123,13 +130,30 @@ sub init {
     Lemonldap::NG::Handler::Main->onReload( $self, 'reloadConf' );
 
     # Handler::PSGI::Try initialization
-    return 0 unless ( $self->SUPER::init( $self->localConfig ) );
+    unless ( $self->SUPER::init( $self->localConfig ) ) {
+        $self->logger->error( 'Initialization failed: ' . $self->error );
+        $self->error(
+"Initialization failed! Enable debug logs, reload your web server and catch main error..."
+        );
+        return 0;
+    }
     if ( $self->error ) {
         $self->logger->error( $self->error );
         return 0;
     }
 
-    # Handle requests (other path may be declared in enabled plugins)
+    # Default routes must point to routines declared above
+    $self->defaultAuthRoute('');
+    $self->defaultUnauthRoute('');
+    return 1;
+}
+
+sub setPortalRoutes {
+    my ($self) = @_;
+    $self->authRoutes(
+        { GET => {}, POST => {}, PUT => {}, DELETE => {}, OPTIONS => {} } );
+    $self->unAuthRoutes(
+        { GET => {}, POST => {}, PUT => {}, DELETE => {}, OPTIONS => {} } );
     $self
 
       # "/" or undeclared paths
@@ -140,38 +164,43 @@ sub init {
 
       # psgi.js
       ->addUnauthRoute( 'psgi.js' => 'sendJs', ['GET'] )
-      ->addAuthRoute( 'psgi.js' => 'sendJs', ['GET'] )
+      ->addAuthRoute( 'psgi.js' => 'sendJs',   ['GET'] )
 
       # portal.css
       ->addUnauthRoute( 'portal.css' => 'sendCss', ['GET'] )
-      ->addAuthRoute( 'portal.css' => 'sendCss', ['GET'] )
+      ->addAuthRoute( 'portal.css' => 'sendCss',   ['GET'] )
 
       # lmerror
       ->addUnauthRoute( lmerror => { ':code' => 'lmError' }, ['GET'] )
-      ->addAuthRoute( lmerror => { ':code' => 'lmError' }, ['GET'] )
+      ->addAuthRoute( lmerror => { ':code'   => 'lmError' }, ['GET'] )
 
       # Core REST API
-      ->addUnauthRoute( ping => 'pleaseAuth', ['GET'] )
+      ->addUnauthRoute( ping => 'pleaseAuth',  ['GET'] )
       ->addAuthRoute( ping => 'authenticated', ['GET'] )
 
       # Refresh session
       ->addAuthRoute( refresh => 'refresh', ['GET'] )
 
-      ->addAuthRoute( '*' => 'corsPreflight', ['OPTIONS'] )
+      ->addAuthRoute( '*' => 'corsPreflight',   ['OPTIONS'] )
       ->addUnauthRoute( '*' => 'corsPreflight', ['OPTIONS'] )
 
       # Logout
-      ->addAuthRoute( logout => 'logout', ['GET'] );
+      ->addAuthRoute( logout => 'logout', ['GET'] )
+      ->addUnauthRoute( logout => 'unauthLogout', ['GET'] );
 
     # Default routes must point to routines declared above
     $self->defaultAuthRoute('');
     $self->defaultUnauthRoute('');
     return 1;
+
 }
 
 sub reloadConf {
     my ( $self, $conf ) = @_;
 
+    # Handle requests (other path may be declared in enabled plugins)
+    $self->setPortalRoutes;
+
     # Reinitialize $self->conf
     %{ $self->{conf} } = %{ $self->localConfig };
 
@@ -203,18 +232,22 @@ sub reloadConf {
     {
         my $header = $_;
         my $prm    = $self->conf->{ 'cors' . $_ };
-        $header =~ s/_/-/;
-        $prm    =~ s/\s+//;
-        $cors .= "Access-Control-$header;$prm;";
+        if ( $header and $prm ) {
+            $header =~ s/_/-/;
+            $prm    =~ s/\s+//;
+            $cors .= "Access-Control-$header;$prm;";
+        }
     }
     $self->cors($cors);
     $self->logger->debug( "Initialized CORS headers : " . $self->cors );
 
     # Initialize templateDir
     $self->{templateDir} =
-      $self->conf->{templateDir} . '/' . $self->conf->{portalSkin};
+      $self->conf->{templateDir} . '/' . $self->conf->{portalSkin}
+      if ( $self->conf->{templateDir} and $self->conf->{portalSkin} );
     unless ( -d $self->{templateDir} ) {
-        $self->error("Template dir $self->{templateDir} doesn't exist");
+        $self->error("Template dir $self->{templateDir} doesn't exist")
+          if ref( $self->{templateDir} ) eq 'SCALAR';
         return $self->fail;
     }
     $self->templateDir(
@@ -245,6 +278,11 @@ sub reloadConf {
     }
     $self->conf->{domain} =~ s/^([^\.])/.$1/;
 
+    # Initialize cookie SameSite value
+    $self->cookieSameSite( getSameSite( $self->conf ) );
+    $self->logger->debug(
+        "Cookies will use SameSite=" . $self->cookieSameSite );
+
     # Load menu
     # ---------
     $self->menu( $self->loadPlugin('::Main::Menu') );
@@ -356,11 +394,7 @@ sub reloadConf {
 
     # Clean $req->pdata after authentication
     push @{ $self->endAuth }, sub {
-
-        my $tmp =
-          ( ref( $_[0]->pdata->{keepPdata} ) eq 'ARRAY' )
-          ? $_[0]->pdata->{keepPdata}
-          : [];
+        my $tmp = $_[0]->pdata->{keepPdata} //= [];
         foreach my $k ( keys %{ $_[0]->pdata } ) {
             unless ( grep { $_ eq $k } @$tmp ) {
                 $self->logger->debug("Removing $k from pdata");
@@ -465,6 +499,25 @@ sub findEP {
             }
         }
     }
+    if ( $obj->can('hook') ) {
+        $self->logger->debug("Found hook in $plugin");
+        my $h = $obj->hook;
+        unless ( ref $h and ref($h) eq 'HASH' ) {
+            $self->logger->error('"hook" endpoint must be a hashref, skipped');
+        }
+        else {
+            foreach my $hookname ( keys %$h ) {
+                my $callback = $h->{$hookname};
+                push @{ $self->hook->{$hookname} }, sub {
+                    eval {
+                        $obj->logger->debug(
+                            "Launching ${plugin}::$callback on hook $hookname");
+                    };
+                    $obj->$callback(@_);
+                };
+            }
+        }
+    }
     $self->logger->debug("Plugin $plugin initializated");
 
     # Rules for menu
@@ -498,7 +551,11 @@ sub loadModule {
         $self->error("Unable to build $module object: $@");
         return 0;
     }
-    ( $obj and $obj->init ) or return 0;
+    unless ( $obj and $obj->init ) {
+        $self->error("$module init failed");
+        return 0;
+    }
+
     $self->loadedModules->{$module} = $obj;
     return $obj;
 }
@@ -516,4 +573,25 @@ sub displayError {
         'Portal error, contact your administrator', 500 );
 }
 
+# This helper method builds a rule from a string expression
+# - $rule: rule text
+# - $ruleDesc optional hint of what the rule is for, to display in error message
+# returns undef if the rule syntax was invalid
+sub buildRule {
+    my ( $self, $rule, $ruleDesc ) = @_;
+    if ($ruleDesc) {
+        $ruleDesc = " $ruleDesc ";
+    }
+    else {
+        $ruleDesc = " ";
+    }
+    my $compiledRule =
+      $self->HANDLER->buildSub( $self->HANDLER->substitute($rule) );
+    unless ($compiledRule) {
+        my $error = $self->HANDLER->tsv->{jail}->error || '???';
+        $self->logger->error( "Bad" . $ruleDesc . "rule: " . $error );
+    }
+    return $compiledRule,;
+}
+
 1;
