@@ -3,13 +3,13 @@ package Lemonldap::NG::Portal::Plugins::CheckUser;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
-  PE_BADCREDENTIALS
-  PE_TOKENEXPIRED
   PE_NOTOKEN
+  PE_TOKENEXPIRED
   PE_MALFORMEDUSER
+  PE_BADCREDENTIALS
 );
 
-our $VERSION = '2.0.9';
+our $VERSION = '2.0.10';
 
 extends qw(
   Lemonldap::NG::Portal::Main::Plugin
@@ -18,7 +18,6 @@ extends qw(
 );
 
 # INITIALIZATION
-
 has ott => (
     is      => 'rw',
     lazy    => 1,
@@ -29,13 +28,14 @@ has ott => (
         return $ott;
     }
 );
-has idRule                    => ( is => 'rw', default => sub { 1 } );
-has displayEmptyValuesRule    => ( is => 'rw', default => sub { 0 } );
-has displayEmptyHeadersRule   => ( is => 'rw', default => sub { 0 } );
-has displayPersistentInfoRule => ( is => 'rw', default => sub { 0 } );
-has unrestrictedUsersRule     => ( is => 'rw', default => sub { 0 } );
-has sorted                    => ( is => 'rw', default => sub { 0 } );
-has merged                    => ( is => 'rw', default => '' );
+has unrestrictedUsersRule      => ( is => 'rw', default => sub { 0 } );
+has displayEmptyValuesRule     => ( is => 'rw', default => sub { 0 } );
+has displayEmptyHeadersRule    => ( is => 'rw', default => sub { 0 } );
+has displayPersistentInfoRule  => ( is => 'rw', default => sub { 0 } );
+has displayComputedSessionRule => ( is => 'rw', default => sub { 0 } );
+has idRule                     => ( is => 'rw', default => sub { 1 } );
+has sorted                     => ( is => 'rw', default => sub { 0 } );
+has merged                     => ( is => 'rw', default => '' );
 
 sub hAttr {
     $_[0]->{conf}->{checkUserHiddenAttributes} . ' '
@@ -49,8 +49,8 @@ sub persistentAttrs {
 
 sub init {
     my ($self) = @_;
-    $self->addAuthRoute( checkuser => 'check', ['POST'] );
-    $self->addAuthRouteWithRedirect( checkuser => 'display', ['GET'] );
+    $self->addAuthRoute( checkuser => 'check', ['POST'] )
+      ->addAuthRouteWithRedirect( checkuser => 'display', ['GET'] );
 
     # Parse checkUser rules
     $self->idRule(
@@ -84,11 +84,19 @@ sub init {
     $self->unrestrictedUsersRule(
         $self->p->buildRule(
             $self->conf->{checkUserUnrestrictedUsersRule},
-            'checkUserUnrestrictedUsers'
+            'checkUserUnrestrictedUsersRule'
         )
     );
     return 0 unless $self->unrestrictedUsersRule;
 
+    $self->displayComputedSessionRule(
+        $self->p->buildRule(
+            $self->conf->{checkUserDisplayComputedSession},
+            'checkUserdisplayComputedSession'
+        )
+    );
+    return 0 unless $self->displayComputedSessionRule;
+
     # Init. other options
     $self->sorted( $self->conf->{impersonationRule}
           || $self->conf->{contextSwitchingRule} );
@@ -98,40 +106,24 @@ sub init {
     return 1;
 }
 
-# RUNNING METHOD
+# RUNNING METHODS
 sub display {
-    my ( $self, $req ) = @_;
-    my ( $attrs, $array_attrs ) = ( {}, [] );
+    my ( $self,  $req )         = @_;
+    my ( $attrs, $array_attrs ) = ( $req->userData, [] );
 
     $self->logger->debug("Display current session data...");
     $self->userLogger->info("Using spoofed SSO groups if exist")
       if ( $self->conf->{impersonationRule} );
-    $attrs = $req->userData;
 
-    $attrs = $self->_removePersistentAttributes($attrs)
+    $attrs =
+      $self->_removeKeys( $attrs, $self->persistentAttrs,
+        'Remove persistent session attributes...' )
       unless $self->displayPersistentInfoRule->( $req, $req->userData );
 
-    # Create an array of hashes for template loop
-    $self->logger->debug("Delete hidden or empty attributes");
-    if ( $self->displayEmptyValuesRule->( $req, $req->userData ) ) {
-        foreach my $k ( sort keys %$attrs ) {
-
-            # Ignore hidden attributes
-            push @$array_attrs, { key => $k, value => $attrs->{$k} }
-              unless ( $self->hAttr =~ /\b$k\b/ );
-        }
-    }
-    else {
-        foreach my $k ( sort keys %$attrs ) {
-
-            # Ignore hidden attributes and empty values
-            push @$array_attrs, { key => $k, value => $attrs->{$k} }
-              unless ( $self->hAttr =~ /\b$k\b/ or !$attrs->{$k} );
-        }
-    }
-
+    # Create an array of hashes and dispatch attributes for template loop
     # ARRAY_REF = [ A_REF GROUPS, A_REF MACROS, A_REF OTHERS ]
-    $array_attrs = $self->_splitAttributes($array_attrs);
+    $array_attrs = $self->_dispatchAttributes(
+        $self->_createArray( $req, $attrs, $req->userData ) );
 
     # Display form
     my $params = {
@@ -151,7 +143,7 @@ sub display {
             : ''
         )
     };
-    return $self->sendJSONresponse( $req, $params ) if ( $req->wantJSON );
+    return $self->sendJSONresponse( $req, $params ) if $req->wantJSON;
 
     # Display form
     return $self->p->sendHtml( $req, 'checkuser', params => $params );
@@ -160,8 +152,9 @@ sub display {
 sub check {
     my ( $self, $req ) = @_;
     my ( $attrs, $array_attrs, $array_hdrs ) = ( {}, [], [] );
-    my $msg = my $auth = my $compute = '';
-    my $unUser = $self->unrestrictedUsersRule->( $req, $req->userData ) || 0;
+    my $msg           = my $auth = my $computed = '';
+    my $savedUserData = $req->userData;
+    my $unUser = $self->unrestrictedUsersRule->( $req, $savedUserData ) || 0;
 
     # Check token
     if ( $self->ottRule->( $req, {} ) ) {
@@ -189,14 +182,13 @@ sub check {
             LOGIN     => '',
             TOKEN     => $token,
         };
-        return $self->p->sendJSONresponse( $req, $params )
-          if ( $req->wantJSON );
+        return $self->p->sendJSONresponse( $req, $params ) if $req->wantJSON;
         return $self->p->sendHtml( $req, 'checkuser', params => $params )
           if $msg;
     }
 
     ## Check user session datas
-    # Use submitted attribute if exists
+    # Use submitted attributes if exists
     my $url  = $req->param('url')  || '';
     my $user = $req->param('user') || '';
 
@@ -204,7 +196,7 @@ sub check {
         $user  = '';
         $attrs = {};
         return $self->p->sendError( $req, 'Malformed user', 400 )
-          if ( $req->wantJSON );
+          if $req->wantJSON;
         return $self->p->sendHtml(
             $req,
             'checkuser',
@@ -228,7 +220,7 @@ sub check {
     if ( !$user or $user eq $req->{user} ) {
         $self->userLogger->info("checkUser requested for himself");
         $self->userLogger->info("Using spoofed SSO groups if exist")
-          if ( $self->conf->{impersonationRule} );
+          if $self->conf->{impersonationRule};
         $attrs = $req->userData;
         $user  = $req->{user};
     }
@@ -266,12 +258,15 @@ sub check {
             $req->{user} = $user;
             $self->userLogger->info(
                 "No session found in DB. Compute userData...");
-            $attrs   = $self->_userData($req);
-            $compute = 1;
+            $attrs    = $self->_userData($req);
+            $computed = 1;
         }
 
         # Check identities rule
-        $self->logger->info("\"$user\" is an unrestricted user!") if $unUser;
+        $self->logger->info( '"'
+              . $savedUserData->{ $self->conf->{whatToTrace} }
+              . '" is an unrestricted user!' )
+          if $unUser;
         unless ( $unUser || $self->idRule->( $req, $attrs ) ) {
             $self->userLogger->warn(
                 "checkUser requested for an invalid user ($user)");
@@ -288,67 +283,66 @@ sub check {
         $attrs       = {};
     }
     else {
-        $msg   = 'checkUser' . $self->merged;
-        $attrs = $self->_removePersistentAttributes($attrs)
-          unless $self->displayPersistentInfoRule->( $req, $req->userData );
+        $msg = 'checkUser' . $self->merged;
+        $attrs =
+          $self->_removeKeys( $attrs, $self->persistentAttrs,
+            'Remove persistent session attributes...' )
+          unless $self->displayPersistentInfoRule->( $req, $savedUserData );
 
-        if ($compute) {
-            $msg = 'checkUserComputeSession';
-            if ( $self->conf->{impersonationRule} ) {
-                $self->logger->debug("Map real attributes...");
-                my %realAttrs = map {
-                    ( "$self->{conf}->{impersonationPrefix}$_" => $attrs->{$_} )
-                } keys %$attrs;
-                $attrs = { %$attrs, %realAttrs };
+        if ($computed) {
+            if ( $self->displayComputedSessionRule->( $req, $savedUserData ) ) {
+                $msg = 'checkUserComputedSession';
+                if ( $self->conf->{impersonationRule} ) {
+                    $self->logger->debug("Map real attributes...");
+                    my %realAttrs = map {
+                        ( "$self->{conf}->{impersonationPrefix}$_" =>
+                              $attrs->{$_} )
+                    } keys %$attrs;
+                    $attrs = { %$attrs, %realAttrs };
 
-                # Compute groups and macros with real and spoofed attributes
-                $self->logger->debug(
-                    "Compute groups and macros with real and spoofed attributes"
-                );
-                $req->sessionInfo($attrs);
-                delete $req->sessionInfo->{groups};
-                $req->steps( [ $self->p->groupsAndMacros, 'setLocalGroups' ] );
-                if ( my $error = $self->p->process($req) ) {
-                    $self->logger->debug("Process returned error: $error");
-                    return $req->error($error);
+                    # Compute groups and macros with real and spoofed attributes
+                    $self->logger->debug(
+"Compute groups and macros with real and spoofed attributes"
+                    );
+                    $req->sessionInfo($attrs);
+                    delete $req->sessionInfo->{groups};
+                    $req->steps(
+                        [ $self->p->groupsAndMacros, 'setLocalGroups' ] );
+                    if ( my $error = $self->p->process($req) ) {
+                        $self->logger->debug("Process returned error: $error");
+                        return $req->error($error);
+                    }
                 }
             }
-        }
-
-        # Create an array of hashes for template loop
-        $self->logger->debug("Delete hidden or empty attributes");
-        if ( $self->displayEmptyValuesRule->( $req, $req->userData ) ) {
-            foreach my $k ( sort keys %$attrs ) {
-
-                # Ignore hidden attributes
-                push @$array_attrs, { key => $k, value => $attrs->{$k} }
-                  unless ( $self->hAttr =~ /\b$k\b/ );
-            }
-        }
-        else {
-            foreach my $k ( sort keys %$attrs ) {
-
-                # Ignore hidden attributes and empty values
-                push @$array_attrs, { key => $k, value => $attrs->{$k} }
-                  unless ( $self->hAttr =~ /\b$k\b/ or !$attrs->{$k} );
+            else {
+                $attrs = {};
+                $msg   = 'checkUserNoSessionFound';
             }
         }
 
+        # Create an array of hashes and dispatch attributes for template loop
         # ARRAY_REF = [ A_REF GROUPS, A_REF MACROS, A_REF OTHERS ]
-        $array_attrs = $self->_splitAttributes($array_attrs);
+        $array_attrs = $self->_dispatchAttributes(
+            $self->_createArray( $req, $attrs, $savedUserData ) );
+    }
+
+    if ( $self->p->checkXSSAttack( 'CheckUser URL', $url ) ) {
+        $url  = '';
+        $auth = 'VHnotFound';
     }
 
     # Check if user is allowed to access submitted URL and compute headers
     if ( $url and %$attrs ) {
 
         # Check url format
-        $url = $self->_urlFormat($url);
+        my $originalUrl;
+        ( $url, $originalUrl ) = $self->_resolveURL( $req, $url );
 
         # User is allowed ?
         $self->logger->debug(
-"checkUser requested for user: $attrs->{ $self->{conf}->{whatToTrace} } and URL: $url"
+"checkUser requested for user: $attrs->{ $self->{conf}->{whatToTrace} } and URL: $url | alias: $originalUrl"
         );
-        $auth = $self->_authorization( $req, $url, $attrs );
+        $auth = $self->_authorization( $req, $originalUrl, $attrs );
         if ( $auth >= 0 ) {
             $auth = $auth ? "allowed" : "forbidden";
             $self->logger->debug(
@@ -356,7 +350,8 @@ sub check {
                   . "$auth to access to $url" );
 
             # Return VirtualHost headers
-            $array_hdrs = $self->_headers( $req, $url, $attrs );
+            $array_hdrs =
+              $self->_headers( $req, $originalUrl, $attrs, $savedUserData );
         }
         else {
             $auth = 'VHnotFound';
@@ -370,17 +365,14 @@ sub check {
 
     # TODO:
     my $params = {
-        PORTAL    => $self->conf->{portal},
-        MAIN_LOGO => $self->conf->{portalMainLogo},
-        SKIN      => $self->p->getSkin($req),
-        LANGS     => $self->conf->{showLanguages},
-        MSG       => $msg,
-        ALERTE    => ( $msg eq 'checkUser' ? 'alert-info' : 'alert-warning' ),
-        LOGIN     => $user,
-        URL       => (
-              $self->p->checkXSSAttack( 'URL', $url ) ? ""
-            : $url
-        ),
+        PORTAL      => $self->conf->{portal},
+        MAIN_LOGO   => $self->conf->{portalMainLogo},
+        SKIN        => $self->p->getSkin($req),
+        LANGS       => $self->conf->{showLanguages},
+        MSG         => $msg,
+        ALERTE      => ( $msg eq 'checkUser' ? 'alert-info' : 'alert-warning' ),
+        LOGIN       => $user,
+        URL         => $url,
         ALLOWED     => $auth,
         ALERTE_AUTH => $alert_auth,
         HEADERS     => $array_hdrs,
@@ -388,27 +380,35 @@ sub check {
         MACROS      => $array_attrs->[1],
         GROUPS      => $array_attrs->[0],
         TOKEN       => (
-            $self->ottRule->( $req, {} ) ? $self->ott->createToken()
+              $self->ottRule->( $req, {} )
+            ? $self->ott->createToken()
             : ''
         )
     };
-    return $self->p->sendJSONresponse( $req, $params ) if ( $req->wantJSON );
+    return $self->p->sendJSONresponse( $req, $params ) if $req->wantJSON;
 
     # Display form
     return $self->p->sendHtml( $req, 'checkuser', params => $params );
 }
 
-sub _urlFormat {
-    my ( $self, $url ) = @_;
-
-    $url = 'http://' . $url unless ( $url =~ m#^https?://[^/]*.*#i );
-    my ( $proto, $vhost, $appuri ) = $url =~ m#^(https?://)([^/]*)(.*)#i;
+sub _resolveURL {
+    my ( $self, $req, $url ) = @_;
+    my ($proto) = $url =~ m#^(https?://).*#i;
+    my ( $vhost, $appuri ) = $url =~ m#^(?:https?://)?([^/]*)(.*)#i;
     my ($port) = $vhost =~ m#^.+(:\d+)$#;
     $port ||= '';
     $vhost =~ s/:\d+$//;
     $vhost .= $self->conf->{domain} unless ( $vhost =~ /\./ );
+    $proto =
+      $self->p->HANDLER->_isHttps( $req, $vhost ) ? 'https://' : 'http://'
+      unless $proto;
+    $self->logger->debug( 'VHost is ' . uc( ( split( /:/, $proto ) )[0] ) );
+    my $originalVhost = $self->p->HANDLER->resolveAlias($vhost);
 
-    return lc("$proto$vhost$port") . "$appuri";
+    return (
+        lc("$proto$vhost$port") . $appuri,
+        lc("$proto$originalVhost$port") . $appuri
+    );
 }
 
 sub _userData {
@@ -425,12 +425,9 @@ sub _userData {
       : push @$steps, 'setLocalGroups';
     $req->steps($steps);
     if ( my $error = $self->p->process($req) ) {
-        if ( $error == PE_BADCREDENTIALS ) {
-            $self->userLogger->warn(
-                    'checkUser requested for an invalid user ('
-                  . $req->{user}
-                  . ")" );
-        }
+        $self->userLogger->warn(
+            'checkUser requested for an invalid user (' . $req->{user} . ")" )
+          if ( $error == PE_BADCREDENTIALS );
         $self->logger->debug("Process returned error: $error");
         return $req->error(PE_BADCREDENTIALS);
     }
@@ -446,7 +443,8 @@ sub _userData {
     $req->sessionInfo->{authenticationLevel} = $realAuthLevel;
     delete $req->sessionInfo->{groups};
 
-    $req->steps( [ 'setSessionInfo', $self->p->groupsAndMacros, 'setLocalGroups' ] );
+    $req->steps(
+        [ 'setSessionInfo', $self->p->groupsAndMacros, 'setLocalGroups' ] );
     if ( my $error = $self->p->process($req) ) {
         $self->logger->debug("CheckUser: Process returned error: $error");
         return $req->error($error);
@@ -479,29 +477,88 @@ sub _authorization {
 }
 
 sub _headers {
-    my ( $self, $req, $uri, $attrs ) = @_;
+    my ( $self, $req, $uri, $attrs, $savedUserData ) = @_;
     my ($vhost) = $uri =~ m#^https?://([^/]*).*#;
 
     $vhost =~ s/:\d+$//;
     $req->{env}->{HTTP_HOST} = $vhost;
     $self->p->HANDLER->headersInit( $self->{conf} );
+    my $headers = $self->p->HANDLER->checkHeaders( $req, $attrs );
+
+    # Remove hidden headers relative to VHost if required
+    unless ( $self->unrestrictedUsersRule->( $req, $savedUserData ) ) {
+        my $keysToRemove = '';
+        $keysToRemove = '__all__'
+          if exists $self->conf->{checkUserHiddenHeaders}->{$vhost};
+        $keysToRemove = $self->conf->{checkUserHiddenHeaders}->{$vhost}
+          if ( $keysToRemove
+            && $self->conf->{checkUserHiddenHeaders}->{$vhost} =~ /\w+/ );
+
+        if ( $keysToRemove eq '__all__' ) {
+            $self->logger->debug(
+                "Overwrite for VirtualHost: $vhost ALL valued header(s)...");
+            @$headers = map {
+                $_->{value} =~ /\w+/
+                  ? { key => $_->{key}, value => '******' }
+                  : $_
+            } @$headers;
+        }
+        else {
+            $self->logger->debug(
+                "Mask hidden header(s) for VirtualHost: $vhost");
+            my $hash = { map { $_->{key} => $_->{value} } @$headers };
+            $hash = $self->_removeKeys( $hash, $keysToRemove,
+                "Overwrite valued \"$keysToRemove\" header(s)...", 1 );
+            @$headers = (
+                map { { key => $_, value => $hash->{$_} } }
+                sort keys %$hash
+            );
+        }
+    }
+
+    # Remove empty headers if required
+    unless ( $self->displayEmptyHeadersRule->( $req, $savedUserData ) ) {
+        $self->logger->debug("Remove empty headers...");
+        @$headers = grep $_->{value} =~ /.+/, @$headers;
+    }
     $self->logger->debug(
         "Return \"$attrs->{ $self->{conf}->{whatToTrace} }\" headers");
-    return $self->p->HANDLER->checkHeaders( $req, $attrs )
-      if ( $self->displayEmptyHeadersRule->( $req, $req->userData ) );
 
-    $self->logger->debug("Remove empty headers");
-    my @headers = grep $_->{value} =~ /.+/,
-      @{ $self->p->HANDLER->checkHeaders( $req, $attrs ) };
-
-    return \@headers;
+    return $headers;
 }
 
-sub _splitAttributes {
+sub _createArray {
+    my ( $self, $req, $attrs, $userData ) = @_;
+    my $array_attrs = [];
+
+    if ( $self->displayEmptyValuesRule->( $req, $userData ) ) {
+        $self->logger->debug("Delete hidden attributes");
+        foreach my $k ( sort keys %$attrs ) {
+
+            # Ignore hidden attributes
+            push @$array_attrs, { key => $k, value => $attrs->{$k} }
+              unless ( $self->hAttr =~ /\b$k\b/ );
+        }
+    }
+    else {
+        $self->logger->debug("Delete hidden and empty attributes");
+        foreach my $k ( sort keys %$attrs ) {
+
+            # Ignore hidden attributes and empty values
+            push @$array_attrs, { key => $k, value => $attrs->{$k} }
+              unless ( $self->hAttr =~ /\b$k\b/ or !$attrs->{$k} );
+        }
+    }
+
+    return $array_attrs;
+}
+
+sub _dispatchAttributes {
     my ( $self, $attrs ) = @_;
     my ( $grps, $mcrs, $others ) = ( [], [], [] );
     my $macros = $self->{conf}->{macros};
-    $self->logger->debug("Dispatching attributes...");
+
+    $self->logger->debug("Dispatch attributes...");
     while (@$attrs) {
         my $element = shift @$attrs;
         $self->logger->debug( "Processing element: $element->{key} => "
@@ -529,9 +586,9 @@ sub _splitAttributes {
 
     # Sort real and spoofed attributes if required
     if ( $self->sorted ) {
-        $self->logger->debug('Dispatching real and spoofed attributes...');
+        $self->logger->debug('Sort real and spoofed attributes...');
         my ( $realAttrs, $spoofedAttrs ) = ( [], [] );
-        my $prefix = "$self->{conf}->{impersonationPrefix}";
+        my $prefix = $self->{conf}->{impersonationPrefix};
         while (@$others) {
             my $element = shift @$others;
             $self->logger->debug( "Processing attribute $element->{key} => "
@@ -546,15 +603,26 @@ sub _splitAttributes {
         }
         @$others = ( @$spoofedAttrs, @$realAttrs );
     }
+
     return [ $grps, $mcrs, $others ];
 }
 
-sub _removePersistentAttributes {
-    my ( $self, $attrs ) = @_;
-    my $regex = join '|',      split /\s+/, $self->persistentAttrs;
+sub _removeKeys {
+    my ( $self, $attrs, $hidden, $msg, $mask ) = @_;
+    my $regex = join '|',      split /\s+/, $hidden;
     my @keys  = grep /$regex/, keys %$attrs;
-    $self->logger->debug("Remove persistent session attributes");
-    delete @$attrs{@keys};
+
+    $self->logger->debug($msg);
+    if ($mask) {
+        $self->userLogger->info('Hide some headers');
+        foreach (@keys) {
+            $attrs->{$_} = '******' if $attrs->{$_} =~ /\w+/;
+        }
+    }
+    else {
+        $self->userLogger->info('Remove some headers');
+        delete @$attrs{@keys};
+    }
 
     return $attrs;
 }
